﻿using System;
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Helpers;


namespace PKAMAII
{
  public class PKAMAII : IDouble2DoubleHandler {
    [HandlerParameter( true, "10", Min = "1", Max = "32767", Step = "1" )]
    public int Period { get; set; }

    [HandlerParameter( true, "0.666", Min = "0", Max = "1", Step = "0.001" )]
    public double FEndF { get; set; }

    [HandlerParameter( true, "0.0645", Min = "0", Max = "1", Step = "0.0001" )]
    public double SEndF { get; set; }

    public IList<double> Execute( IList<double> source ) {
      int count = source.Count, i, period = Period, buffSize = Math.Min( count, period );
      if (count == 0)
        return new double[0];
      var result = new double[ count ];
      var dNoise = new double[ buffSize ];

      result[ 0 ] = source[ 0 ];
      for(i = 1; i < buffSize; i++) {
        dNoise[ i ] = Math.Abs( source[ i ] - source[ i - 1 ] );
        result[ i ] = source[ i ];
      }

      int k = i;
      var prev = i > 0 ? result[ i - 1 ] : 0;
      for(; i < count; i++) {
        if(k == buffSize)
          k = 0;
        dNoise[ k++ ] = Math.Abs( source[ i ] - source[ i - 1 ] );
        double noise = 0;
        for(int j = 0; j < buffSize; j++)
          noise += dNoise[ j ];

        var signal = Math.Abs( source[ i ] - source[ i - period ] );
        var effRatio = noise > 0 ? signal / noise : 0;
        //var feRatio = FEndF * effRatio + SEndF * (1 - effRatio);
        var feRatio = SEndF + effRatio * (FEndF - SEndF);
        var smoothF = feRatio * feRatio;
        //result[ i ] = source[ i ] * smoothF + (1 - smoothF) * result[ i - 1 ];
        result[ i ] = prev = prev + smoothF * (source[ i ] - prev);
      }
      return result;
    }
  }
}